# encoding: utf-8

import pytest
import responses
import requests
import os

from efipay import EfiPay
from efipay import Constants

from efipay import MissingParametersError
from efipay import UnauthorizedError


class TestEfiPay():
    auth_success = {u'access_token': u'token', u'token_type': u'Bearer',
                    u'expires_in': 600, u'refresh_token': u'refresh'}

    charge = {u'code': 200, u'data': {u'status': u'new', u'created_at': u'2016-01-08 11:05:21',
                                      u'custom_id': None, u'total': 3000, u'charge_id': 1724}}


    def test_remove_placeholders(self):
        endpoints = EfiPay(options)({})
        fixed_route = endpoints.remove_placeholders(
            Constants.APIS['CHARGES']['ENDPOINTS']['detail_charge']['route'], {'id': 1})
        assert fixed_route == '/charge/1'

        fixed_route = endpoints.remove_placeholders(
            Constants.APIS['CHARGES']['ENDPOINTS']['create_charge']['route'], {})
        assert fixed_route == '/charge'

        fixed_route = endpoints.remove_placeholders(
            Constants.APIS['CHARGES']['ENDPOINTS']['update_carnet_parcel']['route'], {'id': 1, 'parcel': 2})
        assert fixed_route == '/carnet/1/parcel/2'

        with pytest.raises(MissingParametersError) as raised:
            fixed_route = endpoints.remove_placeholders(
                Constants.APIS['CHARGES']['ENDPOINTS']['update_parcel']['route'], {'id': 1})
        assert 'ExceptionInfo MissingParametersError(\'parcel\')' in str(raised)

    def test_query_string(self):
        endpoints = Gerencianet({})

        qs = endpoints.query_string(
            {'age': 78, 'name': 'Gorbadoc', 'lastname': 'Oldbuck'})
        assert all(x in qs for x in ['lastname=Oldbuck', 'age=78', 'name=Gorbadoc', '&'])



# encoding: utf-8

from __future__ import unicode_literals
from six import iteritems

import json
import re
import requests
import inspect
import base64
import os

from .constants import Constants
from .exceptions import MissingParametersError
from .exceptions import UnauthorizedError
from .exceptions import EndpointError
from .exceptions import CertificateError
from .version import *

from functools import partial

class Endpoints(object):

    def __init__(self, options):
        super(Endpoints, self).__init__()
        self.token = None
        self.options = options

    def __getattr__(self, name): 

        if name in Constants.APIS['PIX']['ENDPOINTS']:
            self.endpoints = Constants.APIS['PIX']['ENDPOINTS']
            self.urls = Constants.APIS['PIX']['URL']
        elif name in Constants.APIS['OPEN-FINANCE']['ENDPOINTS']:
            self.endpoints = Constants.APIS['OPEN-FINANCE']['ENDPOINTS']
            self.urls = Constants.APIS['OPEN-FINANCE']['URL']
        elif name in Constants.APIS['PAYMENTS']['ENDPOINTS']:
            self.endpoints = Constants.APIS['PAYMENTS']['ENDPOINTS']
            self.urls = Constants.APIS['PAYMENTS']['URL']
        elif name in Constants.APIS['OPENING-ACCOUNTS']['ENDPOINTS']:
            self.endpoints = Constants.APIS['OPENING-ACCOUNTS']['ENDPOINTS']
            self.urls = Constants.APIS['OPENING-ACCOUNTS']['URL']
        else:
            self.endpoints = Constants.APIS['CHARGES']['ENDPOINTS']
            self.urls =  Constants.APIS['CHARGES']['URL']
            self.options['certificate'] = None
        self.get_url()
        return partial( self.request, self.endpoints[name])

    def request(self, settings, **kwargs):

        if(self.base_url == ""):
            return EndpointError(404)

        oauth = self.authenticate()

        if(oauth == 200):
            params = {} if 'params' not in kwargs else kwargs['params']
            body = {} if 'body' not in kwargs else kwargs['body']
            headers = {} if 'headers' not in kwargs else kwargs['headers']

            response = self.send(settings, params, body, headers)

            try:
                response.json()
            except:
                return '{\'code\': ' + str(response.status_code) + '}'
            else:
                return response.json()
        elif(oauth == 404):
            return CertificateError(404)
        else:
            return UnauthorizedError(oauth)
            

    def send(self, settings, params, body, headersComplement):
        url = self.build_url(settings['route'], params)

        if(self.options['certificate']):
            headers = {
                'Authorization': 'Bearer {token}'.format(token=self.token['access_token']),
                'Content-Type': 'application/json',
                'api-sdk': 'python-{v}'.format(v=VERSION)
            }

            for (key,value) in headersComplement.items():
                headers[key] = value

            if 'partner_token' in self.options:
                headers['partner-token'] = self.options['partner_token']
            cert=self.options['certificate']
            return requests.request(settings['method'],url, headers=headers, data=json.dumps(body), cert=cert)
        else:
            headers = {
                'accept': 'application/json',
                'api-sdk': 'python-{v}'.format(v=VERSION),
                'Authorization': 'Bearer {token}'.format(token=self.token['access_token'])
            }
            if 'partner_token' in self.options:
                headers['partner-token'] = self.options['partner_token']
            return requests.request(settings['method'], url, json=body, headers=headers)

    def authenticate(self):
        url = self.build_url(self.endpoints['authorize']['route'], {})             
        
        if(self.options['certificate']):
            auth = base64.b64encode((f"{self.options['client_id']}:{self.options['client_secret']}").encode()).decode()
            payload = "{\r\n    \"grant_type\": \"client_credentials\"\r\n}"
            headers = {
                'Authorization': f"Basic {auth}",
                'Content-Type': 'application/json'
            }
            if 'partner_token' in self.options:
                headers['partner-token'] = self.options['partner_token']
            cert=self.options['certificate']
            if(os.path.exists(cert)):
                response = requests.post(url, headers=headers, data=payload, cert=cert)
            else:
                return 404
        else:
            headers = {
                'accept': 'application/json',
                'api-sdk': 'python-{v}'.format(v=VERSION)
            }
            if 'partner_token' in self.options:
                headers['partner-token'] = self.options['partner_token']
            auth = (self.options['client_id'], self.options['client_secret'])
            auth_body =  {'grant_type': 'client_credentials'}
            response = requests.post(url, headers=headers, auth=auth, json=auth_body)

        if response.status_code == 200:
            json =  response.json()
            self.token = json

        return response.status_code
            

    def get_url(self):
        self.base_url = self.urls['sandbox']
        if 'sandbox' in self.options:
            self.base_url = self.urls['sandbox'] if self.options[
                'sandbox'] else self.urls['production']

    def build_url(self, route, params):
        params = {} if params is None else params
        route = self.remove_placeholders(route, params)
        return self.complete_url(route, params)

    def remove_placeholders(self, route, params):
        regex = r'\:(\w+)'
        match = re.findall(regex, route)
        if match:
            for attr in match:
                if attr not in params:
                    raise MissingParametersError(attr)
                route = route.replace(':' + attr, str(params[attr]))
                del params[attr]

        return route

    def query_string(self, params):
        mapped = []
        for (p, value) in iteritems(params):
            mapped.append('{attr}={value}'.format(attr=p, value=value))
        return '&'.join(mapped)

    def complete_url(self, route, params):
        qs = self.query_string(params)

        if len(qs) == 0:
            return '{base}{route}'.format(base=self.base_url, route=route)

        return '{base}{route}?{qs}'.format(base=self.base_url, route=route, qs=qs)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


dependencies = [dependency.strip()
                for dependency in open("requirements.txt").readlines()]

setup(
    name='efipay',

    version='1.0.0',

    description='Module for integration with EfiPay API',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/efipay/sdk-python-apis-efi',

    # Author details
    author='Consultoria Técnica',
    author_email='consultoria@sejaefi.com.br',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.11'
    ],

    # What does your project relate to?
    keywords='payment EfiPay',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'test': ['pytest-cov', 'pytest', 'responses'],
    },

    package_data={
        # If any package contains *.txt or *.rst files, include them:
        '': ['requirements.txt', '.md', 'LICENSE']
    },

    include_package_data=True,

    install_requires=dependencies,
)



# encoding: utf-8

class Constants(object):
    APIS = {
        'CHARGES': {
            'URL': {
                'production': 'https://cobrancas.api.efipay.com.br/v1',
                'sandbox': 'https://cobrancas-h.api.efipay.com.br/v1'
            },
            'ENDPOINTS': {
                'authorize': {
                    'route': '/authorize',
                    'method': 'post'
                },
                'create_charge': {
                    'route': '/charge',
                    'method': 'post'
                },
                'create_one_step_charge': {
                    'route': '/charge/one-step',
                    'method': 'post'
                },
                'detail_charge': {
                    'route': '/charge/:id',
                    'method': 'get'
                },
                'update_charge_metadata': {
                    'route': '/charge/:id/metadata',
                    'method': 'put'
                },
                'update_billet': {
                    'route': '/charge/:id/billet',
                    'method': 'put'
                },
                'define_pay_method': {
                    'route': '/charge/:id/pay',
                    'method': 'post'
                },
                'cancel_charge': {
                    'route': '/charge/:id/cancel',
                    'method': 'put'
                },
                'create_carnet': {
                    'route': '/carnet',
                    'method': 'post'
                },
                'detail_carnet': {
                    'route': '/carnet/:id',
                    'method': 'get'
                },
                'update_carnet_parcel': {
                    'route': '/carnet/:id/parcel/:parcel',
                    'method': 'put'
                },
                'update_carnet_metadata': {
                    'route': '/carnet/:id/metadata',
                    'method': 'put'
                },
                'get_notification': {
                    'route': '/notification/:token',
                    'method': 'get'
                },
                'list_plans': {
                    'route': '/plans',
                    'method': 'get'
                },
                'create_plan': {
                    'route': '/plan',
                    'method': 'post'
                },
                'delete_plan': {
                    'route': '/plan/:id',
                    'method': 'delete'
                },
                'create_subscription': {
                    'route': '/plan/:id/subscription',
                    'method': 'post'
                },
                'create_one_step_subscription': {
                    'route': '/plan/:id/subscription/one-step',
                    'method': 'post'
                },
                'create_one_step_subscription_link': {
                    'route': '/plan/:id/subscription/one-step/link',
                    'method': 'post'
                },
                'detail_subscription': {
                    'route': '/subscription/:id',
                    'method': 'get'
                },
                'define_subscription_pay_method': {
                    'route': '/subscription/:id/pay',
                    'method': 'post'
                },
                'cancel_subscription': {
                    'route': '/subscription/:id/cancel',
                    'method': 'put'
                },
                'update_subscription_metadata': {
                    'route': '/subscription/:id/metadata',
                    'method': 'put'
                },
                'create_subscription_history': {
                    'route': '/subscription/:id/history',
                    'method': 'post'
                },
                'send_subscription_link_email': {
                    'route': '/charge/:id/subscription/resend',
                    'method': 'post'
                },
                'get_installments': {
                    'route': '/installments',
                    'method': 'get'
                },
                'send_billet_email': {
                    'route': '/charge/:id/billet/resend',
                    'method': 'post'
                },
                'create_charge_history': {
                    'route': '/charge/:id/history',
                    'method': 'post'
                },
                'send_carnet_email': {
                    'route': '/carnet/:id/resend',
                    'method': 'post'
                },
                'send_carnet_parcel_email': {
                    'route': '/carnet/:id/parcel/:parcel/resend',
                    'method': 'post'
                },
                'create_carnet_history': {
                    'route': '/carnet/:id/history',
                    'method': 'post'
                },
                'cancel_carnet': {
                    'route': '/carnet/:id/cancel',
                    'method': 'put'
                },
                'cancel_carnet_parcel': {
                    'route': '/carnet/:id/parcel/:parcel/cancel',
                    'method': 'put'
                },
                'create_one_step_link': {
                    'route': '/charge/one-step/link',
                    'method': 'post'
                },
                'define_link_pay_method': {
                    'route': '/charge/:id/link',
                    'method': 'post'
                },
                'update_charge_link': {
                    'route': '/charge/:id/link',
                    'method': 'put'
                },
                'send_link_email': {
                    'route': '/charge/:id/link/resend',
                    'method': 'post'
                },
                'update_plan': {
                    'route': '/plan/:id',
                    'method': 'put'
                },
                'define_balance_sheet_billet': {
                    'route': '/charge/:id/balance-sheet',
                    'method': 'post'
                },
                'settle_charge': {
                    'route': '/charge/:id/settle',
                    'method': 'put'
                },
                'settle_carnet': {
                    'route': '/carnet/:id/settle',
                    'method': 'put'
                },
                'settle_carnet_parcel': {
                    'route': '/carnet/:id/parcel/:parcel/settle',
                    'method': 'put'
                }
            }
        },
        'PIX': {
            'URL': {
                'production': 'https://pix.api.efipay.com.br',
                'sandbox': 'https://pix-h.api.efipay.com.br'
            },
            'ENDPOINTS': {
                'authorize': {
                    'route': '/oauth/token',
                    'method': 'post'
                },
                'pix_config_webhook': {
                    'route': '/v2/webhook/:chave',
                    'method': 'put'
                },
                'pix_detail_webhook': {
                    'route': '/v2/webhook/:chave',
                    'method': 'get'
                },
                'pix_list_webhook': {
                    'route': '/v2/webhook',
                    'method': 'get'
                },
                'pix_delete_webhook': {
                    'route': '/v2/webhook/:chave',
                    'method': 'delete'
                },
                'pix_create_charge': {
                    'route': '/v2/cob/:txid',
                    'method': 'put'
                },
                'pix_create_immediate_charge': {
                    'route': '/v2/cob',
                    'method': 'post'
                },
                'pix_detail_charge': {
                    'route': '/v2/cob/:txid',
                    'method': 'get'
                },
                'pix_update_charge': {
                    'route': '/v2/cob/:txid',
                    'method': 'patch'
                },
                'pix_list_charges': {
                    'route': '/v2/cob',
                    'method': 'get'
                },
                'pix_generate_qrcode': {
                    'route': '/v2/loc/:id/qrcode',
                    'method': 'get'
                },
                'pix_devolution': {
                    'route': '/v2/pix/:e2eId/devolucao/:id',
                    'method': 'put'
                },
                'pix_detail_devolution': {
                    'route': '/v2/pix/:e2eId/devolucao/:id',
                    'method': 'get'
                },
                'pix_send': {
                    'route': '/v2/gn/pix/:idEnvio',
                    'method': 'put'
                },
                'pix_send_detail': {
                    'route': '/v2/gn/pix/enviados/:e2eid',
                    'method': 'get'
                },
                'pix_send_list': {
                    'route': '/v2/pix/:e2eId',
                    'method': 'get'
                },
                'pix_detail_received': {
                    'route': '/v2/pix/:e2eId',
                    'method': 'get'
                },
                'pix_received_list': {
                    'route': '/v2/pix',
                    'method': 'get'
                },
                'pix_list_received': {
                    'route': '/v2/pix',
                    'method': 'get'
                },
                'pix_create_location': {
                    'route': '/v2/loc',
                    'method': 'post'
                },
                'pix_location_list': {
                    'route': '/v2/loc',
                    'method': 'get'
                },
                'pix_detail_location': {
                    'route': '/v2/loc/:id',
                    'method': 'get'
                },
                'pix_unlink_txid_location': {
                    'route': '/v2/loc/:id/txid',
                    'method': 'delete'
                },
                'pix_create_evp': {
                    'route': '/v2/gn/evp',
                    'method': 'post'
                },
                'pix_list_evp': {
                    'route': '/v2/gn/evp',
                    'method': 'get'
                },
                'pix_delete_evp': {
                    'route': '/v2/gn/evp/:chave',
                    'method': 'delete'
                },
                'get_account_balance': {
                    'route': '/v2/gn/saldo',
                    'method': 'get'
                },
                'update_account_config': {
                    'route': '/v2/gn/config',
                    'method': 'put'
                },
                'list_account_config': {
                    'route': '/v2/gn/config',
                    'method': 'get'
                },
                'pix_create_due_charge': {
                    'route': '/v2/cobv/:txid',
                    'method': 'put'
                },
                'pix_update_due_charge': {
                    'route': '/v2/cobv/:txid',
                    'method': 'patch'
                },
                'pix_detail_due_charge': {
                    'route': '/v2/cobv/:txid',
                    'method': 'get'
                },
                'pix_list_due_charges': {
                    'route': '/v2/cobv/',
                    'method': 'get'
                },
                'create_report': {
                    'route': '/v2/gn/relatorios/extrato-conciliacao',
                    'method': 'post'
                },
                'detail_report': {
                    'route': '/v2/gn/relatorios/:id',
                    'method': 'get'
                },
                'pix_split_detail_charge': {
                    'route': '/v2/gn/split/cob/:txid',
                    'method': 'get'
                },
                'pix_split_link_charge': {
                    'route': '/v2/gn/split/cob/:txid/vinculo/:splitConfigId',
                    'method': 'put'
                },
                'pix_split_unlink_charge': {
                    'route': '/v2/gn/split/cob/:txid/vinculo',
                    'method': 'delete'
                },
                'pix_split_detail_due_charge': {
                    'route': '/v2/gn/split/cobv/:txid',
                    'method': 'get'
                },
                'pix_split_link_due_charge': {
                    'route': '/v2/gn/split/cobv/:txid/vinculo/:splitConfigId',
                    'method': 'put'
                },
                'pix_split_unlink_due_charge': {
                    'route': '/v2/gn/split/cobv/:txid/vinculo',
                    'method': 'delete'
                },
                'pix_split_config_id': {
                    'route': '/v2/gn/split/config/:id',
                    'method': 'put'
                },
                'pix_split_config': {
                    'route': '/v2/gn/split/config',
                    'method': 'post'
                },
                'pix_split_detail_config': {
                    'route': '/v2/gn/split/config/:id',
                    'method': 'get'
                }
            }
        },
        'OPEN-FINANCE': {
            'URL': {
                'production': 'https://openfinance.api.efipay.com.br/v1',
                'sandbox': 'https://openfinance-h.api.efipay.com.br/v1'
            },
            'ENDPOINTS': {
                'authorize': {
                    'route': '/oauth/token',
                    'method': 'post'
                },
                'of_config_update': {
                    'route': '/config',
                    'method': 'put'
                },
                'of_config_detail': {
                    'route': '/config',
                    'method': 'get'
                },
                'of_devolution_pix': {
                    'route': '/pagamentos/pix/:identificadorPagamento/devolver',
                    'method': 'post'
                },
                'of_list_participants': {
                    'route': '/participantes/',
                    'method': 'get'
                }, 
                'of_start_pix_payment': {
                    'route': '/pagamentos/pix',
                    'method': 'post'
                },
                'of_list_pix_payment': {
                    'route': '/pagamentos/pix',
                    'method': 'get'
                }
            }
        },
        'PAYMENTS': {
            'URL': {
                'production': 'https://pagarcontas.api.efipay.com.br/v1',
                'sandbox': ''
            },
            'ENDPOINTS':{
                'authorize': {
                    'route': '/oauth/token',
                    'method': 'post'
                },
                'pay_detail_bar_code': {
                    'route': '/codBarras/:codBarras',
                    'method': 'get'
                },
                'pay_request_bar_code': {
                    'route': '/codBarras/:codBarras',
                    'method': 'post'
                },
                'pay_detail_payment': {
                    'route': '/:idPagamento',
                    'method': 'get'
                },
                'pay_list_payments': {
                    'route': '/resumo',
                    'method': 'get'
                }
            }
        },
        'OPENING-ACCOUNTS': {
            'URL': {
                'production': 'https://abrircontas.api.efipay.com.br/v1',
                'sandbox': ''
            },
            'ENDPOINTS': {
                'authorize': {
                    'route': '/oauth/token',
                    'method': 'post'
                },
                'create_account': {
                    'route': '/cadastro/conta-simplificada',
                    'method': 'post'
                },
                'get_account_credentials': {
                    'route': '/cadastro/conta-simplificada/:idContaSimplificada/credenciais',
                    'method': 'get'
                },
                'get_account_certificate': {
                    'route': '/cadastro/conta-simplificada/:idContaSimplificada/certificado',
                    'method': 'get'
                },
                'account_config_webhook': {
                    'route': '/cadastro/webhook',
                    'method': 'post'
                },
                'account_list_webhook': {
                    'route': '/cadastro/webhooks',
                    'method': 'get'
                },
                'account_detail_webhook': {
                    'route': '/cadastro/webhook/:identificadorWebhook',
                    'method': 'get'
                },
                'account_delete_webhook': {
                    'route': '/cadastro/webhook/:identificadorWebhook',
                    'method': 'delete'
                }
            }
        }
    }



# encoding: utf-8

class EfiPayError(Exception):

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)

class MissingParametersError(EfiPayError):

    def __init__(self, parameter):
        self.parameter = parameter
        message = 'Missing required parameter {parameter}'.format(parameter= self.parameter)
        super(MissingParametersError, self).__init__(message)


class UnauthorizedError(EfiPayError):

    def __init__(self, status):
        message = "{ 'Status': " + str(status) + ", 'Message': 'Could not authenticate. Please make sure you are using correct credentials and if you are using then in the correct environment.' } "
        super(UnauthorizedError, self).__init__(message)


class EndpointError(EfiPayError):

    def __init__(self, status):
        message = "{ 'Status': " + str(status) + ", 'Message': 'This method isn't available in sandbox.' } "
        super(EndpointError, self).__init__(message)

class CertificateError(EfiPayError):

    def __init__(self, status):
        message = "{ 'Status': " + str(status) + ", 'Message': 'Certificate not found.' }"
        super(CertificateError, self).__init__(message)